#!/usr/bin/perl 
#Usage: mkskyflat.pl zen pa filtB filtR 
#Usage: mkskyflat.pl hh mm ss dd mm ss pa filtB filtR
#
# zen: looks up LST at sunset/sunrise and RA = LST - 00:30. DEC=27 deg
# pab par desired PA for Blue and Red cameras in degrees
#
#filtB = SDT_Uspec U-BESSEL B-BESSEL V-BESSEL g-SLOAN r-SLOAN
#filtR = V-BESSEL R-BESSEL I-BESSEL r-SLOAN i-SLOAN z-SLOAN Y-FAN F972N20
#
my $infile;
my $rahh;
my $ramm;
my $rass;
my $decd;
my $decm;
my $decs;
my $pa;
my $filtB;
my $filtR;
my $currdatetime;
my $mth; my $day; my $mh; my $mm; my $ms; my $yy;
my %bscl = (
	'V' => '0.0',
	'S' => '+0.55',
	'U' => '+0.60',
	'B' => '0.0',
	'r' => '0.0',
	'g' => '-0.35',
	    );	
#my %rscl = (
#	'V' => '0.0',
#	'R' => '-0.1',
#	'r' => '-0.1',
#	'I' => '-0.3',
#	'i' => '-0.3',
#	'z' => '0.0',
#	'Y' => '+0.8',
#	'F' => '+1.2',
#	    );	
my %rscl = (
	'V' => '0.0',
	'R' => '0.0',
	'r' => '0.0',
	'I' => '-0.1',
	'i' => '-0.1',
	'z' => '0.0',
	'Y' => '+0.6',
	'F' => '+0.9',
	    );	
my %month = (
	'Jan' => '01',
	'Feb' => '02',
	'Mar' => '03',
	'Apr' => '04',
	'May' => '05',
	'Jun' => '06',
	'Jul' => '07',
	'Aug' => '08',
	'Sep' => '09',
	'Oct' => '10',
	'Nov' => '11',
	'Dec' => '12',
	    );	
my %twi = (
	'Jan' => '77',
	'Feb' => '74',
	'Mar' => '74',
	'Apr' => '77',
	'May' => '85',
	'Jun' => '91',
	'Jul' => '88',
	'Aug' => '80',
	'Sep' => '74',
	'Oct' => '73',
	'Nov' => '76',
	'Dec' => '78',
	    );	

my $flag = 0;

$nargs = scalar(@ARGV);
printf ("Number of arguments is %d\n",$nargs);

if(($nargs != 9)&&($nargs != 4)&&($nargs!=11)&&($nargs!=6)) {
	printf ("Command line syntax may be one of the following:\n ./mkskyflat1.pl [RAhh] [RAmm] [RAss] [DECdd] [DECmm] [DECss] [PA(deg)] [LBC-Blue filter] [LBC-Red filter] [LBC-Blue exptime] [LBC-Red exptime] or\n ./mkskyflat1.pl [RAhh] [RAmm] [RAss] [DECdd] [DECmm] [DECss] [PA(deg)] [LBC-Blue filter] [LBC-Red filter] or \n./mkskyflat1.pl [eve|morn] [PA(deg)] [LBC-Blue filter] [LBC-Red filter] [LBC-Blue exptime] [LBC-Red exptime] or \n ./mkskyflat1.pl [eve|morn] [PA(deg)] [LBC-Blue filter] [LBC-Red filter] \nwhere the filters may be designated by the first letters of their names:\nFor LBC-Blue: \n[S]DT_Uspec, [U]-BESSEL, [B]-BESSEL, [V]-BESSEL, [g]-SLOAN and [r]-SLOAN.\nFor LBC-Red: \n[V]-BESSEL, [R]-BESSEL, [I]-BESSEL, [r]-SLOAN, [i]-SLOAN, [z]-SLOAN, [Y]-FAN and [F]972N20\n");
	$flag = 2;
	}

if($nargs == 11) {
$flag = 3;
$rahh = @ARGV[0];
$ramm = @ARGV[1];
$rass = @ARGV[2];
$decd = @ARGV[3];
$decm = @ARGV[4];
$decs = @ARGV[5];
$pa = @ARGV[6];
$filtB = @ARGV[7];
$filtR = @ARGV[8];
$exptB = @ARGV[9];
$exptR = @ARGV[10];
$radeg = 15.*($rahh + (($ramm + ($rass/60.))/60.));
# account for negative declinations
$tmp = reverse($decd);
$sign = chop $tmp;
if ($sign eq "-") {
        $decdeg = $decd + ((-1.*$decm + (-1.*$decs/60.))/60.);
        }
else {
        $decdeg = $decd + (($decm + ($decs/60.))/60.);
        }
$dec = $decd;
}

if($nargs == 6) {
$flag = 3;
$twi = @ARGV[0];
$pa = @ARGV[1];
$filtB = @ARGV[2];
$filtR = @ARGV[3];
$exptB = @ARGV[4];
$exptR = @ARGV[5];
#printf ("nargs = %d twi=%s pa=%f filtB = %s filtR=%s\n",$nargs,$twi,$pa,$filtB,$filtR);
system "date > datefile";
open(IN3,datefile)|| die "cannot open file datefile for reading: $!";
while(<IN3>) {
$currdatetime = $_;
chomp($currdatetime);
#printf "current date and time are: %s\n",$currdatetime;
}
if($currdatetime =~ /\S*\s*(\S*)\s*(\d*)\s*(\d+):(\d+):(\d+)\s*\S*\s*(\d*)/) {
	$mth = $1;
	$day = $2;
	$mh = $3;
	$mm = $4;
	$ms = $5;
	$yy = $6;
	#printf("test %d %s %d %d %d %d %d\n", $month{$mth}, $mth, $day, $mh, $mm, $ms, $yy);
	}
open(OUTS,">skycalcinp") || die "cannot open file skycalcinp for writing: $!";
printf OUTS ("g\ny %d %d %d a\nQ\n",$yy,$month{$mth},$day);
#
if(substr($twi,0,1) eq "e") {
system "./skycalc < skycalcinp | grep 'LMST at evening' > skycalcout";
#system "skycalc < skycalcinp | grep 'LMST at evening' > skycalcout";
$factor = -1.0;
}
elsif(substr($twi,0,1) eq "m") {
system "./skycalc < skycalcinp | grep 'LMST at morning' > skycalcout";
#system "skycalc < skycalcinp | grep 'LMST at morning' > skycalcout";
$factor = 1.0;
}
open(IN4,skycalcout)||die "cannot open file skycalcout for reading: $!";
while(<IN4>) {
	$lmst = $_;
	chomp($lmst);
	#printf "skycalcoutput = %s\n", $lmst;
	if($lmst =~ /\S*\s*\S*\s*\d*\s*\d*\s*\S*\s*\S*\s*\S*\s*\S*\s*\S*\s*(\d*)\s*(\d*)\s*/) 
		{
		$lh = $1;
		$lm = $2;
		# RA is approx LST at sunrise/sunset(hence factor= +/- 1) minus 15 minutes
		$radeg = 15.*(($lh+($lm/60.))+$factor*(($twi{$mth})/60.) - 0.25);
		if ($radeg < 0) { 
			$radeg = $radeg+360;}
  		if ($radeg >= 360) {
			$radeg = $radeg-360;}
		#printf ("%d %d %d %f\n",$lh,$lm,$twi{$mth},$radeg);
		}
   	}
$decdeg = 27.00;
$rahh = $lh; #just LST hour at twilight, not converted to LST at sunrise/sunset
$ramm = $lm; #just LST minute at twilight, not converted to LST at sunrise/sunset
$dec = $decdeg;
}

if($nargs == 9) {
$rahh = @ARGV[0];
$ramm = @ARGV[1];
$rass = @ARGV[2];
$decd = @ARGV[3];
$decm = @ARGV[4];
$decs = @ARGV[5];
$pa = @ARGV[6];
$filtB = @ARGV[7];
$filtR = @ARGV[8];
#$exptB = @ARGV[9];
#$exptR = @ARGV[10];
$radeg = 15.*($rahh + (($ramm + ($rass/60.))/60.));
# account for negative declinations
$tmp = reverse($decd);
$sign = chop $tmp;
if ($sign eq "-") {
        $decdeg = $decd + ((-1.*$decm + (-1.*$decs/60.))/60.);
        }
else {
        $decdeg = $decd + (($decm + ($decs/60.))/60.);
        }
$dec = $decd;
}


if($nargs == 4) {
$twi = @ARGV[0];
$pa = @ARGV[1];
$filtB = @ARGV[2];
$filtR = @ARGV[3];
#$exptB = @ARGV[4];
#$exptR = @ARGV[5];
#printf ("nargs = %d twi=%s pa=%f filtB = %s filtR=%s\n",$nargs,$twi,$pa,$filtB,$filtR);
system "date > datefile";
open(IN3,datefile)|| die "cannot open file datefile for reading: $!";
while(<IN3>) {
$currdatetime = $_;
chomp($currdatetime);
#printf "current date and time are: %s\n",$currdatetime;
}
if($currdatetime =~ /\S*\s*(\S*)\s*(\d*)\s*(\d+):(\d+):(\d+)\s*\S*\s*(\d*)/) {
	$mth = $1;
	$day = $2;
	$mh = $3;
	$mm = $4;
	$ms = $5;
	$yy = $6;
	#printf("test %d %s %d %d %d %d %d\n", $month{$mth}, $mth, $day, $mh, $mm, $ms, $yy);
	}
open(OUTS,">skycalcinp") || die "cannot open file skycalcinp for writing: $!";
printf OUTS ("g\ny %d %d %d a\nQ\n",$yy,$month{$mth},$day);
#
if(substr($twi,0,1) eq "e") {
system "./skycalc < skycalcinp | grep 'LMST at evening' > skycalcout";
#system "skycalc < skycalcinp | grep 'LMST at evening' > skycalcout";
$factor = -1.0;
}
elsif(substr($twi,0,1) eq "m") {
system "./skycalc < skycalcinp | grep 'LMST at morning' > skycalcout";
#system "skycalc < skycalcinp | grep 'LMST at morning' > skycalcout";
$factor = 1.0;
}
open(IN4,skycalcout)||die "cannot open file skycalcout for reading: $!";
while(<IN4>) {
	$lmst = $_;
	chomp($lmst);
	#printf "skycalcoutput = %s\n", $lmst;
	if($lmst =~ /\S*\s*\S*\s*\d*\s*\d*\s*\S*\s*\S*\s*\S*\s*\S*\s*\S*\s*(\d*)\s*(\d*)\s*/) 
		{
		$lh = $1;
		$lm = $2;
		# RA is approx LST at sunrise/sunset(hence factor= +/- 1) minus 15 minutes
		$radeg = 15.*(($lh+($lm/60.))+$factor*(($twi{$mth})/60.) - 0.25);
		if ($radeg < 0) { 
			$radeg = $radeg+360;}
  		if ($radeg >= 360) {
			$radeg = $radeg-360;}
		#printf ("%d %d %d %f\n",$lh,$lm,$twi{$mth},$radeg);
		}
   	}
$decdeg = 27.00;
$rahh = $lh;
$ramm = $lm;
$dec = $decdeg;
}


if(($nargs == 9)||($nargs == 4)||($nargs==11)||($nargs==6)) {
if((substr($filtB,0,1) ne "S")&&(substr($filtB,0,1) ne "U")&&(substr($filtB,0,1) ne "B")&&(substr($filtB,0,1) ne "V")&&(substr($filtB,0,1) ne "r")&&(substr($filtB,0,1) ne "g")) {
	printf ("%s is not available in LBC-Blue. ",$filtB);
	print "Please enter at least the first letter of one of the following LBC-Blue filters: [S]DT_Uspec, [U]-BESSEL, [B]-BESSEL, [V]-BESSEL, [g]-SLOAN and [r]-SLOAN\n";
	$flag = 2;
	}
if((substr($filtR,0,1) ne "V")&&(substr($filtR,0,1) ne "R")&&(substr($filtR,0,1) ne "I")&&(substr($filtR,0,1) ne "r")&&(substr($filtR,0,1) ne "i")&&(substr($filtR,0,1) ne "z")&&(substr($filtR,0,1) ne "Y")&&(substr($filtR,0,1) ne "F")) {
	printf ("%s is not available in LBC-Red. ",$filtR);
	print "Please enter at least the first letter of one of the following LBC-Red filters: [V]-BESSEL, [R]-BESSEL, [I]-BESSEL, [r]-SLOAN, [i]-SLOAN, [z]-SLOAN, [Y]-FAN and [F]972N20\n";
	$flag = 2;
	}
}

if (substr($filtB,0,1) eq "S") {
	$fb = "S";
	$filtB = "SDT_Uspec";
	}
if (substr($filtB,0,1) eq "U") {
	$fb = "U";
	$filtB = "U-BESSEL";
	}
if (substr($filtB,0,1) eq "B") {
	$fb = "B";
	$filtB = "B-BESSEL";
	}
if (substr($filtB,0,1) eq "V") {
	$fb = "V";
	$filtB = "V-BESSEL";
	}
if (substr($filtB,0,1) eq "g") {
	$fb = "g";
	$filtB = "g-SLOAN";
	}
if (substr($filtB,0,1) eq "r") {
	$fb = "r";
	$filtB = "r-SLOAN";
	}
#
if (substr($filtR,0,1) eq "V") {
	$fr = "V";
	$filtR = "V-BESSEL";
	}
if (substr($filtR,0,1) eq "R") {
	$fr = "R";
	$filtR = "R-BESSEL";
	}
if (substr($filtR,0,1) eq "I") {
	$fr = "I";
	$filtR = "I-BESSEL";
	}
if (substr($filtR,0,1) eq "r") {
	$fr = "r";
	$filtR = "r-SLOAN";
	}
if (substr($filtR,0,1) eq "i") {
	$fr = "i";
	$filtR = "i-SLOAN";
	}
if (substr($filtR,0,1) eq "z") {
	$fr = "z";
	$filtR = "z-SLOAN";
	}
if (substr($filtR,0,1) eq "Y") {
	$fr = "Y";
	$filtR = "Y-FAN";
	}
if (substr($filtR,0,1) eq "F") {
	$fr = "F";
	$filtR = "F972N20";
	}

printf("bscl = %0.3f\t rscl = %0.3f\n",$bscl{$fb},$rscl{$fr});
if ((($bscl{$fb} - $rscl{$fr}) >= 0)&& ($flag !=3)) {
	$exptB = 10**($bscl{$fb} - $rscl{$fr});
	$exptR = 1;
	}
if ((($bscl{$fb} - $rscl{$fr}) < 0)&& ($flag !=3)) {
	$exptB = 1;
	$exptR = 10**($rscl{$fr} - $bscl{$fb});
	}

printf("B time = %0.3f\t R time = %0.3f\n\n",$exptB,$exptR);

if((abs($bscl{$fb}-$rscl{$fr}) >= 0.47)&&($flag != 3)) {
	printf("WARNING!!! \nExposure time estimates differ by more than a factor 3. Could a combination of filters with more similar times be found?\n");
}

if($flag != 2) {
	# determine name of skyflat OB and test OB
	$outtemp1 = "SkyFlat_" . "$fb" . "$fr" . "_r" . "$rahh" . "$ramm" . "d" . "$dec" . "_pa" . "$pa" . "_5.ob";
	open(OUT1,">$outtemp1") || die "cannot open file $outtemp1 for writing: $!";
#
        $outtemp2 = "SkyFlatTest_" . "$fb" . "$fr" . "_r" . "$rahh" . "$ramm" . "d" . "$dec" . "_pa" . "$pa" . ".ob";
	open(OUT2,">$outtemp2") || die "cannot open file $outtemp2 for writing: $!";

	$obname = "SkyFlat_" . "$fb" . "$fr" . "_pa" . "$pa";
	$obtestname =  "SkyFlatTest_" . "$fb" . "$fr" . "_pa" . "$pa";

	printf ("\nRA(deg) = %.3f\t DEC(deg) = %.3f \n",$radeg,$decdeg);
	printf ("Position Angle for both LBC-Blue and LBC-Red = %.0f\n",$pa);
	printf ("LBC-Blue filter = %s\t LBC-Red filter = %s \n",$filtB,$filtR);
	printf ("SkyFlat OB will be %s  Test OB will be %s \n",$outtemp1,$outtemp2);
	system "sed -e s/obname/$obname/g .skyflat4ob >outtemp1a";
	system "sed -e s/tecra/$radeg/g outtemp1a >outtemp1b";
	system "sed -e s/tecdec/$decdeg/g outtemp1b >outtemp1c";
	system "sed -e s/pangle/$pa/g outtemp1c >outtemp1d";
	system "sed -e s/filterB/$filtB/g outtemp1d >outtemp1e";
	system "sed -e s/filterR/$filtR/g outtemp1e >outtemp1f";
	system "sed -e s/exptimB/$exptB/g outtemp1f >outtemp1g";
	system "sed -e s/exptimR/$exptR/g outtemp1g > $outtemp1";
	
	system "sed -e s/obtestname/$obtestname/g .skyflattestob >outtemp2a";
	system "sed -e s/tecra/$radeg/g outtemp2a >outtemp2b";
	system "sed -e s/tecdec/$decdeg/g outtemp2b >outtemp2c";
	system "sed -e s/pangle/$pa/g outtemp2c >outtemp2d";
	system "sed -e s/filterB/$filtB/g outtemp2d >outtemp2e";
	system "sed -e s/filterR/$filtR/g outtemp2e >outtemp2f";
	system "sed -e s/exptimB/$exptB/g outtemp2f >outtemp2g";
	system "sed -e s/exptimR/$exptR/g outtemp2g >$outtemp2";
	
	system "rm outtemp1a outtemp1b outtemp1c outtemp1d outtemp1e";
	system "rm outtemp2a outtemp2b outtemp2c outtemp2d outtemp2e";
	system "rm outtemp1f outtemp1g outtemp2f outtemp2g";

	}

close(OUT1);
close(OUT2);
